/*
 *  AVRVM, a virtual machine which emulates an Atmel AVR CPU.
 *  Copyright (C) 2010  Joakim Soderlund
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "instructions.h"

int execute(ATMega16 *cpu) {
	int i;
	
	cpu->ir = cpu->pm[cpu->pc];
	cpu->pc++;
	
	for (i = 0; avrvm_instructions[i].filter != 0; i++)
		if ((cpu->ir & avrvm_instructions[i].filter) == avrvm_instructions[i].mask)
			return avrvm_instructions[i].function(cpu);
	
	return -1;
}


int main() {
	int status, fd, i;
	unsigned short *instructions;
	unsigned char buffer[2];
	ATMega16 *cpu;
	struct stat fs;
	
	cpu = get_atmega16();

	if (cpu == NULL) {
		printf("Error allocating memory.\n");
		return -1;
	}
	
	fd = open("code.bin", O_RDONLY);
	
	if (fd < 0) {
		printf("Error opening file \"code.bin\"\n");
		return -1;
	}
	
	status = fstat(fd, &fs);
	
	if ((fs.st_size % 2) != 0) {
		printf("The file is invalid.\n");
		return -1;
	}
	
	
	cpu->pm = malloc(sizeof(unsigned short) * (fs.st_size / 2));
	
	/** Read all instructions. */
	for (i = 0; i < (fs.st_size / 2); i++) {
		status = read(fd, &buffer, 2);
		
		if (status < 0) {
			printf("Error reading from file!\n");
			return -1;
		}
		
		cpu->pm[i] = (((unsigned short) buffer[1]) << 8) + buffer[0];
	}
	
	status = 0;
	
	while (status == 0) {
		status = execute(cpu);
	}
	
	printf("\nError: Unkown instruction \"%04X\", aborting execution.\n", cpu->ir);
	
	printf("\nContents of R16: %d\n", cpu->gen[16]);
	printf("Contents of R17: %d\n", cpu->gen[17]);
	printf("Contents of R18: %d\n", cpu->gen[18]);
}

